'use strict';

var _classCallCheck2 = require('babel-runtime/helpers/classCallCheck');

var _classCallCheck3 = _interopRequireDefault(_classCallCheck2);

var _possibleConstructorReturn2 = require('babel-runtime/helpers/possibleConstructorReturn');

var _possibleConstructorReturn3 = _interopRequireDefault(_possibleConstructorReturn2);

var _inherits2 = require('babel-runtime/helpers/inherits');

var _inherits3 = _interopRequireDefault(_inherits2);

exports.__esModule = true;

var _events = require('events');

var _mustache = require('mustache');

var _mustache2 = _interopRequireDefault(_mustache);

var _useragent = require('useragent');

var _readFileRelative = require('read-file-relative');

var _command = require('./command');

var _command2 = _interopRequireDefault(_command);

var _messages = require('../messages');

var _arrayRemove = require('../utils/array-remove');

var _arrayRemove2 = _interopRequireDefault(_arrayRemove);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

// Const
var IDLE_PAGE_TEMPLATE = (0, _readFileRelative.readSync)('../client/browser/idle-page/index.html.mustache');

// Global instance counter used to generate ID's
var instanceCount = 0;

var BrowserConnection = function (_EventEmitter) {
    (0, _inherits3.default)(BrowserConnection, _EventEmitter);

    function BrowserConnection(gateway) {
        (0, _classCallCheck3.default)(this, BrowserConnection);

        var _this = (0, _possibleConstructorReturn3.default)(this, _EventEmitter.call(this));

        _this.HEARTBEAT_TIMEOUT = 2 * 60 * 1000;

        _this.id = ++instanceCount;
        _this.jobQueue = [];
        _this.browserConnectionGateway = gateway;
        _this.userAgent = null;

        _this.closed = false;
        _this.ready = false;
        _this.idle = true;
        _this.switchingToIdle = false;
        _this.heartbeatTimeout = null;

        _this.url = gateway.domain + '/browser/connect/' + _this.id;
        _this.heartbeatUrl = gateway.domain + '/browser/heartbeat/' + _this.id;
        _this.idleUrl = gateway.domain + '/browser/idle/' + _this.id;
        _this.statusUrl = gateway.domain + '/browser/status/' + _this.id;

        _this.on('error', function () {
            return _this.close();
        });

        _this.browserConnectionGateway.startServingConnection(_this);
        return _this;
    }

    BrowserConnection.prototype._waitForHeartbeat = function _waitForHeartbeat() {
        var _this2 = this;

        this.heartbeatTimeout = setTimeout(function () {
            _this2.emit('error', new Error((0, _messages.getText)(_messages.MESSAGE.browserDisconnected, _this2.userAgent)));
        }, this.HEARTBEAT_TIMEOUT);
    };

    BrowserConnection.prototype._popNextTestRunUrl = function _popNextTestRunUrl() {
        if (this.jobQueue.length) {
            var job = this.jobQueue[0];
            var url = job.popNextTestRunUrl();

            if (!job.hasQueuedTestRuns) this.jobQueue.shift();

            return url;
        }

        return null;
    };

    // API

    BrowserConnection.prototype.addJob = function addJob(job) {
        this.jobQueue.push(job);
    };

    BrowserConnection.prototype.removeJob = function removeJob(job) {
        (0, _arrayRemove2.default)(this.jobQueue, job);
    };

    BrowserConnection.prototype.close = function close() {
        this.browserConnectionGateway.stopServingConnection(this);
        clearTimeout(this.heartbeatTimeout);

        this.ready = false;
        this.closed = true;

        this.emit('closed');
    };

    BrowserConnection.prototype.establish = function establish(userAgent) {
        this.ready = true;
        this.userAgent = (0, _useragent.parse)(userAgent).toString();

        this._waitForHeartbeat();
        this.emit('ready');
    };

    BrowserConnection.prototype.heartbeat = function heartbeat() {
        clearTimeout(this.heartbeatTimeout);
        this._waitForHeartbeat();
    };

    BrowserConnection.prototype.renderIdlePage = function renderIdlePage() {
        return _mustache2.default.render(IDLE_PAGE_TEMPLATE, {
            userAgent: this.userAgent,
            statusUrl: this.statusUrl,
            heartbeatUrl: this.heartbeatUrl
        });
    };

    BrowserConnection.prototype.getStatus = function getStatus() {
        if (this.switchingToIdle) {
            this.switchingToIdle = false;
            this.idle = true;
            this.emit('idle');
        }

        var testRunUrl = this._popNextTestRunUrl();

        if (testRunUrl) {
            this.idle = false;
            return { cmd: _command2.default.run, url: testRunUrl };
        }

        this.switchingToIdle = true;
        return { cmd: _command2.default.idle, url: this.idleUrl };
    };

    return BrowserConnection;
}(_events.EventEmitter);

exports.default = BrowserConnection;
module.exports = exports['default'];