'use strict';

var _keys = require('babel-runtime/core-js/object/keys');

var _keys2 = _interopRequireDefault(_keys);

var _regenerator = require('babel-runtime/regenerator');

var _regenerator2 = _interopRequireDefault(_regenerator);

var _asyncToGenerator2 = require('babel-runtime/helpers/asyncToGenerator');

var _asyncToGenerator3 = _interopRequireDefault(_asyncToGenerator2);

var _classCallCheck2 = require('babel-runtime/helpers/classCallCheck');

var _classCallCheck3 = _interopRequireDefault(_classCallCheck2);

exports.__esModule = true;

var _path = require('path');

var _commander = require('commander');

var _pinkie = require('pinkie');

var _pinkie2 = _interopRequireDefault(_pinkie);

var _promisify = require('../utils/promisify');

var _promisify2 = _interopRequireDefault(_promisify);

var _dedent = require('dedent');

var _dedent2 = _interopRequireDefault(_dedent);

var _globby = require('globby');

var _globby2 = _interopRequireDefault(_globby);

var _mkdirp = require('mkdirp');

var _mkdirp2 = _interopRequireDefault(_mkdirp);

var _testcafeBrowserNatives = require('testcafe-browser-natives');

var _readFileRelative = require('read-file-relative');

var _messages = require('../messages');

var _getViewportWidth = require('../utils/get-viewport-width');

var _getViewportWidth2 = _interopRequireDefault(_getViewportWidth);

var _wordWrap = require('../utils/word-wrap');

var _wordWrap2 = _interopRequireDefault(_wordWrap);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var ensureDir = (0, _promisify2.default)(_mkdirp2.default);

var REMOTE_ALIAS_RE = /^(\d*)remote$/;

var DESCRIPTION = (0, _dedent2.default)('\n    In the browser list, you can use aliases (e.g. "ie9", "chrome", etc.) as well as paths to executables.\n\n    To run tests against all installed browsers, use the "all" alias.\n\n    To use a remote browser connection (e.g. to connect a mobile device), specify "remote" as a browser alias. If you\n    need to connect multiple devices, prefix the alias with the number of browsers you want to connect (e.g., "3remote").\n\n    You can use one or more file paths or glob patterns to specify which tests to run.\n\n    More info: http://testcafe.devexpress.com/Documentation/CLI\n');

var CliArgumentParser = function () {
    function CliArgumentParser() {
        (0, _classCallCheck3.default)(this, CliArgumentParser);

        this.program = new _commander.Command('testcafe');

        this.cwd = process.cwd();

        this.src = null;
        this.browsers = null;
        this.filter = null;
        this.remoteCount = 0;
        this.opts = null;

        this._describeProgram();
    }

    CliArgumentParser._parsePortNumber = function _parsePortNumber(value) {
        var number = parseInt(value, 10);

        if (isNaN(value)) throw new Error((0, _messages.getText)(_messages.MESSAGE.portNumberIsNotInteger));

        return number;
    };

    CliArgumentParser._optionValueToRegExp = function _optionValueToRegExp(name, value) {
        if (value === void 0) return value;

        try {
            return new RegExp(value);
        } catch (err) {
            throw new Error((0, _messages.getText)(_messages.MESSAGE.optionValueIsNotValidRegExp, name));
        }
    };

    CliArgumentParser._replaceAllBrowsersAlias = function _replaceAllBrowsersAlias(browserList, browser, allAliases) {
        if (browser === 'all') browserList = browserList.concat(allAliases);else browserList.push(browser);

        return browserList;
    };

    CliArgumentParser._getDescription = function _getDescription() {
        // NOTE: add empty line to workaround commander-forced indentation on the first line.
        return '\n' + (0, _wordWrap2.default)(DESCRIPTION, 2, (0, _getViewportWidth2.default)(process.stdout));
    };

    CliArgumentParser.prototype._describeProgram = function _describeProgram() {
        this.program.version(JSON.parse((0, _readFileRelative.readSync)('../../package.json')).version).usage('[options] <comma-separated-browser-list> <file-or-glob ...>').description(CliArgumentParser._getDescription()).option('-b, --list-browsers', 'output the available browser aliases').option('-r, --reporter <name>', 'specify the reporter type to use').option('-s, --screenshots <path>', 'enable screenshot capturing and specify the path to save the screenshots to').option('-S, --screenshots-on-fails', 'take a screenshot whenever a test fails').option('-q, --quarantine-mode', 'enable the quarantine mode').option('-e, --skip-js-errors', 'make tests not fail when a JS error happens on a page').option('-t, --test <name>', 'run only tests with the specified name').option('-T, --test-grep <pattern>', 'run only tests matching the specified pattern').option('-f, --fixture <name>', 'run only fixtures with the specified name').option('-F, --fixture-grep <pattern>', 'run only fixtures matching the specified pattern').option('--ports <port1,port2>', 'specify custom port numbers').option('--hostname <name>', 'specify the hostname').option('--qr-code', 'outputs QR-code that repeats URLs used to connect the remote browsers')

        // NOTE: these options will be handled by chalk internally
        .option('--color', 'force colors in command line').option('--no-color', 'disable colors in command line');
    };

    CliArgumentParser.prototype._filterAndCountRemotes = function _filterAndCountRemotes(browser) {
        var remoteMatch = browser.match(REMOTE_ALIAS_RE);

        if (remoteMatch) {
            this.remoteCount += parseInt(remoteMatch[1], 10) || 1;
            return false;
        }

        return true;
    };

    CliArgumentParser.prototype._parseFilteringOptions = function _parseFilteringOptions() {
        var _this = this;

        this.opts.testGrep = CliArgumentParser._optionValueToRegExp('--test-grep', this.opts.testGrep);
        this.opts.fixtureGrep = CliArgumentParser._optionValueToRegExp('--fixture-grep', this.opts.fixtureGrep);

        this.filter = function (testName, fixtureName) {

            if (_this.opts.test && testName !== _this.opts.test) return false;

            if (_this.opts.testGrep && !_this.opts.testGrep.test(testName)) return false;

            if (_this.opts.fixture && fixtureName !== _this.opts.fixture) return false;

            if (_this.opts.fixtureGrep && !_this.opts.fixtureGrep.test(fixtureName)) return false;

            return true;
        };
    };

    CliArgumentParser.prototype._parsePorts = function () {
        var ref = (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee() {
            return _regenerator2.default.wrap(function _callee$(_context) {
                while (1) {
                    switch (_context.prev = _context.next) {
                        case 0:
                            if (!this.opts.ports) {
                                _context.next = 4;
                                break;
                            }

                            this.opts.ports = this.opts.ports.split(',').map(CliArgumentParser._parsePortNumber);

                            if (!(this.opts.ports.length < 2)) {
                                _context.next = 4;
                                break;
                            }

                            throw new Error((0, _messages.getText)(_messages.MESSAGE.portsOptionRequiresTwoNumbers));

                        case 4:
                        case 'end':
                            return _context.stop();
                    }
                }
            }, _callee, this);
        }));
        return function _parsePorts() {
            return ref.apply(this, arguments);
        };
    }();

    CliArgumentParser.prototype._parseBrowserList = function () {
        var ref = (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee2() {
            var _this2 = this;

            var browsersArg, installations, allAliases;
            return _regenerator2.default.wrap(function _callee2$(_context2) {
                while (1) {
                    switch (_context2.prev = _context2.next) {
                        case 0:
                            browsersArg = this.program.args[0] || '';
                            _context2.next = 3;
                            return (0, _testcafeBrowserNatives.getInstallations)();

                        case 3:
                            installations = _context2.sent;
                            allAliases = (0, _keys2.default)(installations);

                            this.browsers = browsersArg.split(',').filter(function (browser) {
                                return browser && _this2._filterAndCountRemotes(browser);
                            }).reduce(function (browserList, browser) {
                                return CliArgumentParser._replaceAllBrowsersAlias(browserList, browser, allAliases);
                            }, []);

                        case 6:
                        case 'end':
                            return _context2.stop();
                    }
                }
            }, _callee2, this);
        }));
        return function _parseBrowserList() {
            return ref.apply(this, arguments);
        };
    }();

    CliArgumentParser.prototype._parseFileList = function () {
        var ref = (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee3() {
            var _this3 = this;

            var fileList;
            return _regenerator2.default.wrap(function _callee3$(_context3) {
                while (1) {
                    switch (_context3.prev = _context3.next) {
                        case 0:
                            fileList = this.program.args.slice(1);
                            _context3.next = 3;
                            return (0, _globby2.default)(fileList, {
                                cwd: this.cwd,
                                silent: true,
                                nodir: true
                            });

                        case 3:
                            this.src = _context3.sent;

                            this.src = this.src.map(function (file) {
                                return (0, _path.resolve)(_this3.cwd, file);
                            });

                        case 5:
                        case 'end':
                            return _context3.stop();
                    }
                }
            }, _callee3, this);
        }));
        return function _parseFileList() {
            return ref.apply(this, arguments);
        };
    }();

    CliArgumentParser.prototype._parseScreenshotsPath = function () {
        var ref = (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee4() {
            return _regenerator2.default.wrap(function _callee4$(_context4) {
                while (1) {
                    switch (_context4.prev = _context4.next) {
                        case 0:
                            if (!this.opts.screenshots) {
                                _context4.next = 4;
                                break;
                            }

                            this.opts.screenshots = (0, _path.resolve)(this.cwd, this.opts.screenshots);

                            _context4.next = 4;
                            return ensureDir(this.opts.screenshots);

                        case 4:
                        case 'end':
                            return _context4.stop();
                    }
                }
            }, _callee4, this);
        }));
        return function _parseScreenshotsPath() {
            return ref.apply(this, arguments);
        };
    }();

    CliArgumentParser.prototype.parse = function () {
        var ref = (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee5(argv) {
            return _regenerator2.default.wrap(function _callee5$(_context5) {
                while (1) {
                    switch (_context5.prev = _context5.next) {
                        case 0:
                            this.program.parse(argv);

                            this.opts = this.program.opts();

                            // NOTE: the '-list-browsers' option only lists browsers and immediately exits the app.
                            // Therefore, we don't need to process other arguments.

                            if (this.opts.listBrowsers) {
                                _context5.next = 6;
                                break;
                            }

                            this._parseFilteringOptions();

                            _context5.next = 6;
                            return _pinkie2.default.all([this._parsePorts(), this._parseScreenshotsPath(), this._parseBrowserList(), this._parseFileList()]);

                        case 6:
                        case 'end':
                            return _context5.stop();
                    }
                }
            }, _callee5, this);
        }));
        return function parse(_x) {
            return ref.apply(this, arguments);
        };
    }();

    return CliArgumentParser;
}();

exports.default = CliArgumentParser;
module.exports = exports['default'];