'use strict';

var _classCallCheck2 = require('babel-runtime/helpers/classCallCheck');

var _classCallCheck3 = _interopRequireDefault(_classCallCheck2);

exports.__esModule = true;

var _path = require('path');

var _fs = require('fs');

var _stripBom = require('strip-bom');

var _stripBom2 = _interopRequireDefault(_stripBom);

var _sourceMapSupport = require('source-map-support');

var _sourceMapSupport2 = _interopRequireDefault(_sourceMapSupport);

var _nodeVersion = require('node-version');

var _nodeVersion2 = _interopRequireDefault(_nodeVersion);

var _globals = require('../api/globals');

var _globals2 = _interopRequireDefault(_globals);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var COMMON_API_PATH = (0, _path.join)(__dirname, '../api/common');
var NODE_MODULES_PATH = (0, _path.join)(__dirname, '../../node_modules');
var NODE_VER = parseInt(_nodeVersion2.default.major, 10);
var CWD = process.cwd();

var FIXTURE_RE = /(^|;|\s+)fixture\s*(\(\s*('|").+?\3\s*\)|`.+?`)/;
var TEST_RE = /(^|;|\s+)test\s*(\(\s*('|").+?\3\s*,)/;

var Module = module.constructor;

var ESNextCompiler = function () {
    function ESNextCompiler() {
        (0, _classCallCheck3.default)(this, ESNextCompiler);

        this.sourceMaps = {};
        this.cache = {};

        this._setupSourceMapsSupport();
    }

    ESNextCompiler._getNodeModulesLookupPath = function _getNodeModulesLookupPath(filename) {
        var dir = (0, _path.dirname)(filename);
        var paths = Module._nodeModulePaths(dir);

        // HACK: for npm v2 installations, we need to expose our node_modules,
        // so that Module will be able to require babel-runtime.
        paths.push(NODE_MODULES_PATH);

        return paths;
    };

    ESNextCompiler._getBabelOptions = function _getBabelOptions(filename) {
        // NOTE: lazy load heavy dependencies
        var presetStage2 = require('babel-preset-stage-2');
        var transformRuntime = require('babel-plugin-transform-runtime');
        var presetES2015 = NODE_VER < 4 ? require('babel-preset-es2015-loose') : require('babel-preset-es2015-node4');

        return {
            presets: [presetStage2, presetES2015],
            plugins: [transformRuntime],
            filename: filename,
            sourceMaps: true,
            ast: false,
            babelrc: false,
            highlightCode: false,
            resolveModuleSource: function resolveModuleSource(source) {
                return source === 'testcafe' ? COMMON_API_PATH : source;
            }
        };
    };

    ESNextCompiler._isNodeModulesDep = function _isNodeModulesDep(filename) {
        return (0, _path.relative)(CWD, filename).split(_path.sep).indexOf('node_modules') >= 0;
    };

    ESNextCompiler.prototype._setupSourceMapsSupport = function _setupSourceMapsSupport() {
        var _this = this;

        _sourceMapSupport2.default.install({
            handleUncaughtExceptions: false,

            retrieveSourceMap: function retrieveSourceMap(filename) {
                var map = _this.sourceMaps[filename];

                if (map) return { url: filename, map: map };
            }
        });
    };

    ESNextCompiler.prototype._compileES = function _compileES(code, filename) {
        // NOTE: lazy load heavy dependencies
        var babel = require('babel-core');

        if (this.cache[filename]) return this.cache[filename];

        var opts = ESNextCompiler._getBabelOptions(filename);
        var compiled = babel.transform(code, opts);

        this.cache[filename] = compiled.code;
        this.sourceMaps[filename] = compiled.map;

        return compiled.code;
    };

    ESNextCompiler.prototype._setupRequireHook = function _setupRequireHook(globals) {
        var _this2 = this;

        var origRequireExtension = require.extensions['.js'];

        require.extensions['.js'] = function (mod, filename) {
            // NOTE: remove global API so that it will be unavailable for the dependencies
            globals.remove();

            if (ESNextCompiler._isNodeModulesDep(filename)) origRequireExtension(mod, filename);else {
                var code = (0, _fs.readFileSync)(filename);
                var compiledCode = _this2._compileES((0, _stripBom2.default)(code), filename);

                mod.paths = ESNextCompiler._getNodeModulesLookupPath(filename);

                mod._compile(compiledCode, filename);
            }

            globals.setup();
        };

        return origRequireExtension;
    };

    ESNextCompiler.prototype.canCompile = function canCompile(code, filename) {
        return (/\.js$/.test(filename) && FIXTURE_RE.test(code) && TEST_RE.test(code)
        );
    };

    ESNextCompiler.prototype.compile = function compile(code, filename) {
        var compiledCode = this._compileES(code, filename);
        var mod = new Module(filename, module.parent);
        var globals = new _globals2.default(filename);

        mod.filename = filename;
        mod.paths = ESNextCompiler._getNodeModulesLookupPath(filename);

        globals.setup();

        var origRequireExtension = this._setupRequireHook(globals);

        mod._compile(compiledCode, filename);

        require.extensions['.js'] = origRequireExtension;
        globals.remove();

        return globals.collectedTests;
    };

    ESNextCompiler.prototype.cleanUpCache = function cleanUpCache() {
        this.cache = null;
    };

    return ESNextCompiler;
}();

exports.default = ESNextCompiler;
module.exports = exports['default'];