'use strict';

var _regenerator = require('babel-runtime/regenerator');

var _regenerator2 = _interopRequireDefault(_regenerator);

var _asyncToGenerator2 = require('babel-runtime/helpers/asyncToGenerator');

var _asyncToGenerator3 = _interopRequireDefault(_asyncToGenerator2);

var _stringify = require('babel-runtime/core-js/json/stringify');

var _stringify2 = _interopRequireDefault(_stringify);

var _classCallCheck2 = require('babel-runtime/helpers/classCallCheck');

var _classCallCheck3 = _interopRequireDefault(_classCallCheck2);

var _possibleConstructorReturn2 = require('babel-runtime/helpers/possibleConstructorReturn');

var _possibleConstructorReturn3 = _interopRequireDefault(_possibleConstructorReturn2);

var _inherits2 = require('babel-runtime/helpers/inherits');

var _inherits3 = _interopRequireDefault(_inherits2);

exports.__esModule = true;

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _readFileRelative = require('read-file-relative');

var _mustache = require('mustache');

var _mustache2 = _interopRequireDefault(_mustache);

var _testcafeHammerhead = require('testcafe-hammerhead');

var _command = require('./command');

var _command2 = _interopRequireDefault(_command);

var _type = require('../../reporter/errors/type');

var _type2 = _interopRequireDefault(_type);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

// Const
var TEST_RUN_TEMPLATE = (0, _readFileRelative.readSync)('../../client/test-run/index.js.mustache');
var IFRAME_TEST_RUN_TEMPLATE = (0, _readFileRelative.readSync)('../../client/test-run/iframe.js.mustache');

var TestRun = function (_Session) {
    (0, _inherits3.default)(TestRun, _Session);

    function TestRun(test, browserConnection, screenshotCapturer, opts) {
        (0, _classCallCheck3.default)(this, TestRun);

        var uploadsRoot = _path2.default.dirname(test.fixture.path);

        var _this = (0, _possibleConstructorReturn3.default)(this, _Session.call(this, uploadsRoot));

        _this.running = false;
        _this.unstable = false;

        _this.opts = opts;
        _this.test = test;
        _this.browserConnection = browserConnection;

        _this.isFileDownloading = false;

        // TODO remove it then we move shared data to session storage
        _this.errs = [];
        _this.deferredError = null;
        _this.restartCount = 0;
        _this.nextStep = 0;
        _this.actionTargetWaiting = 0;
        _this.nativeDialogsInfo = null;
        _this.nativeDialogsInfoTimeStamp = 0;
        _this.stepsSharedData = {};
        _this.screenshotCapturer = screenshotCapturer;

        _this.injectable.scripts.push('/testcafe-core.js');
        _this.injectable.scripts.push('/testcafe-ui.js');
        _this.injectable.scripts.push('/testcafe-runner.js');
        _this.injectable.styles.push('/testcafe-ui-styles.css');
        return _this;
    }

    TestRun.prototype._getPayloadScript = function _getPayloadScript() {
        var sharedJs = this.test.fixture.getSharedJs();

        // TODO
        var nextStep = this.actionTargetWaiting ? this.nextStep - 1 : this.nextStep;

        this.actionTargetWaiting = false;

        if (!this.running) {
            this.running = true;
            this.emit('start');
        }

        return _mustache2.default.render(TEST_RUN_TEMPLATE, {
            stepNames: (0, _stringify2.default)(this.test.stepData.names),
            testSteps: this.test.stepData.js,
            sharedJs: sharedJs,
            nextStep: nextStep,
            deferredError: this.deferredError ? (0, _stringify2.default)(this.deferredError) : 'null',
            browserHeartbeatUrl: this.browserConnection.heartbeatUrl,
            browserStatusUrl: this.browserConnection.statusUrl,
            takeScreenshots: this.screenshotCapturer.enabled,
            takeScreenshotsOnFails: this.opts.takeScreenshotsOnFails,
            skipJsErrors: this.opts.skipJsErrors,
            nativeDialogsInfo: (0, _stringify2.default)(this.nativeDialogsInfo)
        });
    };

    TestRun.prototype._getIframePayloadScript = function _getIframePayloadScript(iframeWithoutSrc) {
        var sharedJs = this.test.fixture.getSharedJs();
        var payloadScript = _mustache2.default.render(IFRAME_TEST_RUN_TEMPLATE, {
            sharedJs: sharedJs,
            takeScreenshotsOnFails: this.opts.takeScreenshotsOnFails,
            skipJsErrors: this.opts.skipJsErrors,
            nativeDialogsInfo: (0, _stringify2.default)(this.nativeDialogsInfo)
        });

        return iframeWithoutSrc ? 'var isIFrameWithoutSrc = true;' + payloadScript : payloadScript;
    };

    TestRun.prototype._addError = function () {
        var ref = (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee(err) {
            return _regenerator2.default.wrap(function _callee$(_context) {
                while (1) {
                    switch (_context.prev = _context.next) {
                        case 0:
                            if (err.__sourceIndex !== void 0 && err.__sourceIndex !== null) {
                                err.relatedSourceCode = this.test.sourceIndex[err.__sourceIndex];
                                delete err.__sourceIndex;
                            }

                            _context.prev = 1;
                            _context.next = 4;
                            return this.screenshotCapturer.captureError(err);

                        case 4:
                            err.screenshotPath = _context.sent;
                            _context.next = 9;
                            break;

                        case 7:
                            _context.prev = 7;
                            _context.t0 = _context['catch'](1);

                        case 9:
                            // NOTE: swallow the error silently if we can't take screenshots for some
                            // reason (e.g. we don't have permissions to write a screenshot file).

                            this.errs.push(err);

                        case 10:
                        case 'end':
                            return _context.stop();
                    }
                }
            }, _callee, this, [[1, 7]]);
        }));
        return function _addError(_x) {
            return ref.apply(this, arguments);
        };
    }();

    TestRun.prototype._fatalError = function () {
        var ref = (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee2(err, deferred) {
            return _regenerator2.default.wrap(function _callee2$(_context2) {
                while (1) {
                    switch (_context2.prev = _context2.next) {
                        case 0:
                            if (!deferred) {
                                _context2.next = 3;
                                break;
                            }

                            this.deferredError = err;
                            return _context2.abrupt('return');

                        case 3:
                            _context2.next = 5;
                            return this._addError(err);

                        case 5:
                            this.emit('done');

                        case 6:
                        case 'end':
                            return _context2.stop();
                    }
                }
            }, _callee2, this);
        }));
        return function _fatalError(_x2, _x3) {
            return ref.apply(this, arguments);
        };
    }();

    TestRun.prototype.getAuthCredentials = function getAuthCredentials() {
        return this.test.fixture.authCredentials;
    };

    TestRun.prototype.handleFileDownload = function handleFileDownload() {
        this.isFileDownloading = true;
    };

    TestRun.prototype.handlePageError = function handlePageError(ctx, errMsg) {
        this._fatalError({
            type: _type2.default.pageNotLoaded,
            message: errMsg
        });

        ctx.redirect(this.browserConnection.idleUrl);
    };

    return TestRun;
}(_testcafeHammerhead.Session);

// Service message handlers

exports.default = TestRun;
var ServiceMessages = TestRun.prototype;

ServiceMessages[_command2.default.fatalError] = function (msg) {
    return this._fatalError(msg.err, msg.deferred);
};

ServiceMessages[_command2.default.assertionFailed] = function (msg) {
    return this._addError(msg.err);
};

ServiceMessages[_command2.default.done] = function () {
    this.emit('done');
};

ServiceMessages[_command2.default.setStepsSharedData] = function (msg) {
    this.stepsSharedData = msg.stepsSharedData;
};

ServiceMessages[_command2.default.getStepsSharedData] = function () {
    return this.stepsSharedData;
};

ServiceMessages[_command2.default.setNextStep] = function (msg) {
    this.nextStep = msg.nextStep;
};

ServiceMessages[_command2.default.setActionTargetWaiting] = function (msg) {
    this.actionTargetWaiting = msg.value;
};

ServiceMessages[_command2.default.getAndUncheckFileDownloadingFlag] = function () {
    var isFileDownloading = this.isFileDownloading;

    this.isFileDownloading = false;

    return isFileDownloading;
};

ServiceMessages[_command2.default.uncheckFileDownloadingFlag] = function () {
    this.isFileDownloading = false;
};

ServiceMessages[_command2.default.nativeDialogsInfoSet] = function (msg) {
    if (msg.timeStamp >= this.nativeDialogsInfoTimeStamp) {
        //NOTE: the server can get messages in the wrong sequence if they was sent with a little distance (several milliseconds),
        // we don't take to account old messages
        this.nativeDialogsInfoTimeStamp = msg.timeStamp;
        this.nativeDialogsInfo = msg.info;
    }
};

ServiceMessages[_command2.default.takeScreenshot] = function () {
    var ref = (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee3(msg) {
        return _regenerator2.default.wrap(function _callee3$(_context3) {
            while (1) {
                switch (_context3.prev = _context3.next) {
                    case 0:
                        _context3.prev = 0;
                        _context3.next = 3;
                        return this.screenshotCapturer.captureAction(msg);

                    case 3:
                        return _context3.abrupt('return', _context3.sent);

                    case 6:
                        _context3.prev = 6;
                        _context3.t0 = _context3['catch'](0);
                        return _context3.abrupt('return', null);

                    case 9:
                    case 'end':
                        return _context3.stop();
                }
            }
        }, _callee3, this, [[0, 6]]);
    }));
    return function (_x4) {
        return ref.apply(this, arguments);
    };
}();
module.exports = exports['default'];