'use strict';

var _classCallCheck2 = require('babel-runtime/helpers/classCallCheck');

var _classCallCheck3 = _interopRequireDefault(_classCallCheck2);

var _symbol = require('babel-runtime/core-js/symbol');

var _symbol2 = _interopRequireDefault(_symbol);

exports.__esModule = true;

var _chalk = require('chalk');

var _chalk2 = _interopRequireDefault(_chalk);

var _indentString2 = require('indent-string');

var _indentString3 = _interopRequireDefault(_indentString2);

var _escapeHtml2 = require('escape-html');

var _escapeHtml3 = _interopRequireDefault(_escapeHtml2);

var _moment = require('moment');

var _moment2 = _interopRequireDefault(_moment);

require('moment-duration-format');

var _osFamily = require('os-family');

var _osFamily2 = _interopRequireDefault(_osFamily);

var _wordWrap2 = require('../utils/word-wrap');

var _wordWrap3 = _interopRequireDefault(_wordWrap2);

var _getViewportWidth = require('../utils/get-viewport-width');

var _getViewportWidth2 = _interopRequireDefault(_getViewportWidth);

var _format = require('./errors/format');

var _format2 = _interopRequireDefault(_format);

var _mixinObject = require('mixin-object');

var _mixinObject2 = _interopRequireDefault(_mixinObject);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

// NOTE: we should not expose internal state to
// the plugin, to avoid accidental rewrites.
// Therefore we use symbols to store them.
var stream = (0, _symbol2.default)();
var viewportWidth = (0, _symbol2.default)();
var wordWrapEnabled = (0, _symbol2.default)();
var indent = (0, _symbol2.default)();
var errorDecorator = (0, _symbol2.default)();

var ReporterPluginHost = function () {
    function ReporterPluginHost(plugin, outStream) {
        (0, _classCallCheck3.default)(this, ReporterPluginHost);

        this[stream] = outStream || process.stdout;
        this[viewportWidth] = (0, _getViewportWidth2.default)(this[stream]);
        this[wordWrapEnabled] = false;
        this[indent] = 0;

        var useColors = this[stream] === process.stdout && _chalk2.default.enabled && !plugin.noColors;

        this.chalk = new _chalk2.default.constructor({ enabled: useColors });
        this.moment = _moment2.default;

        this.symbols = _osFamily2.default.win ? { ok: '√', err: '×' } : { ok: '✓', err: '✖' };

        (0, _mixinObject2.default)(this, plugin);

        this[errorDecorator] = this.createErrorDecorator();
    }

    // Error decorator

    ReporterPluginHost.prototype.createErrorDecorator = function createErrorDecorator() {
        var _this = this;

        return {
            'span category': function spanCategory() {
                return '';
            },
            'span step-name': function spanStepName(str) {
                return '"' + str + '"';
            },
            'span user-agent': function spanUserAgent(str) {
                return _this.chalk.gray(str);
            },
            'div screenshot-info': function divScreenshotInfo(str) {
                return str;
            },
            'a screenshot-path': function aScreenshotPath(str) {
                return _this.chalk.underline(str);
            },
            'code': function code(str) {
                return _this.chalk.yellow(str);
            },
            'code step-source': function codeStepSource(str) {
                return _this.chalk.magenta(_this.indentString(str, 4));
            },
            'span code-line': function spanCodeLine(str) {
                return str + '\n';
            },
            'span last-code-line': function spanLastCodeLine(str) {
                return str;
            },
            'code api': function codeApi(str) {
                return _this.chalk.yellow(str);
            },
            'strong': function strong(str) {
                return _this.chalk.cyan(str);
            },
            'a': function a(str) {
                return _this.chalk.yellow('"' + str + '"');
            }
        };
    };

    // String helpers

    ReporterPluginHost.prototype.indentString = function indentString(str, indentVal) {
        return (0, _indentString3.default)(str, ' ', indentVal);
    };

    ReporterPluginHost.prototype.wordWrap = function wordWrap(str, indentVal, width) {
        return (0, _wordWrap3.default)(str, indentVal, width);
    };

    ReporterPluginHost.prototype.escapeHtml = function escapeHtml(str) {
        return (0, _escapeHtml3.default)(str);
    };

    ReporterPluginHost.prototype.formatError = function formatError(err) {
        var prefix = arguments.length <= 1 || arguments[1] === undefined ? '' : arguments[1];

        var maxMsgLength = this[viewportWidth] - this[indent] - prefix.length;
        var msg = (0, _format2.default)(err, this[errorDecorator], maxMsgLength);

        if (this[wordWrapEnabled]) msg = this.wordWrap(msg, prefix.length, maxMsgLength);else msg = this.indentString(msg, prefix.length);

        return prefix + msg.substr(prefix.length);
    };

    // Writing helpers

    ReporterPluginHost.prototype.newline = function newline() {
        this[stream].write('\n');

        return this;
    };

    ReporterPluginHost.prototype.write = function write(text) {
        if (this[wordWrapEnabled]) text = this.wordWrap(text, this[indent], this[viewportWidth]);else text = this.indentString(text, this[indent]);

        this[stream].write(text);

        return this;
    };

    ReporterPluginHost.prototype.useWordWrap = function useWordWrap(use) {
        this[wordWrapEnabled] = use;

        return this;
    };

    ReporterPluginHost.prototype.setIndent = function setIndent(val) {
        this[indent] = val;

        return this;
    };

    // Abstract methods implemented in plugin
    /* eslint-disable no-unused-vars */

    ReporterPluginHost.prototype.reportTaskStart = function reportTaskStart(startTime, userAgents, testCount) {
        throw new Error('Not implemented');
    };

    ReporterPluginHost.prototype.reportFixtureStart = function reportFixtureStart(name, path) {
        throw new Error('Not implemented');
    };

    ReporterPluginHost.prototype.reportTestDone = function reportTestDone(name, errs, durationMs, unstable, screenshotPath) {
        throw new Error('Not implemented');
    };

    ReporterPluginHost.prototype.reportTaskDone = function reportTaskDone(endTime, passed) {
        throw new Error('Not implemented');
    };

    /* eslint-enable no-unused-vars */

    return ReporterPluginHost;
}();

exports.default = ReporterPluginHost;
module.exports = exports['default'];